#!/usr/bin/env python2
# Thomas Levine, ScraperWiki Limited

'''
Local version of ScraperWiki Utils, documentation here:
https://scraperwiki.com/docs/python/python_help_documentation/
'''

logfd = None # This does something on the hosted version

def dumpMessage(d):
    'This does something on the hosted version'
    print(d)

class Error(Exception):
    """All ScraperWiki exceptions are instances of this class
    (usually via a subclass)."""
    pass

class CPUTimeExceededError(Error):
    """CPU time limit exceeded."""
    pass

from .utils import log, scrape, pdftoxml, swimport
import utils, sqlite



from dumptruck import DumpTruck
import re
import os

def _connect(dbname = 'scraperwiki.db'):
  'Initialize the database (again). This is mainly for testing'
  global dt
  dt = DumpTruck(dbname = dbname)

_connect()

def execute(sqlquery, data=[], verbose=1):
    """ Should return list of lists, but returns list of dicts """
    return dt.execute(sqlquery, *data, commit=False)
    # other way [ dict(zip(result["keys"], d))  for d in result["data"] ]

def save(unique_keys, data, table_name="swdata", verbose=2, date=None):
    if not data:
        return
    dt.create_table(data, table_name = table_name, error_if_exists = False)
    if unique_keys != []:
        dt.create_index(unique_keys, table_name, unique = True, if_not_exists = True)
    return dt.insert(data, table_name = table_name)
   
def attach(name, asname=None, verbose=1):
    "This somehow downloads the database from scraperwiki."
    if asname == None:
        asname = re.sub(r'[^a-zA-Z]', '', name)
    os.system('wget -O %s https://scraperwiki.com/scrapers/export_sqlite/%s' % (asname, name))
    dt.execute('attach {0} AS {0}'.format(asname), commit = False)

def commit(verbose=1):
    dt.commit()

def select(sqlquery, data=None, verbose=1):
    sqlquery = "select %s" % sqlquery   # maybe check if select or another command is there already?
    if data == None:
        return dt.execute(sqlquery, commit = False)
    else:
        raise NotImplementedError('Dunno what that data argument does')

def show_tables(dbname=""):
    if dbname != '':
        raise NotImplementedError('Only the main database is implemented')
    response = select('name, sql from sqlite_master where type = "table";')
    return {row['name']: row['sql'] for row in response}

def save_var(name, value, verbose=2):
    return dt.save_var(name, value)

def get_var(name, default=None, verbose=2):
    try:
        return dt.get_var(name)
    except NameError:
        return default



#!/usr/bin/env python2
# utils.py
# David Jones, ScraperWiki Limited
# Thomas Levine, ScraperWiki Limited

'''
Local version of ScraperWiki Utils, documentation here:
https://scraperwiki.com/docs/python/python_help_documentation/
'''

import os
import warnings
import tempfile
import urllib, urllib2
 
def log(message=""):
    '''
    This is useful for profiling the code in the browser,
    but we just print the message in the local version
    '''
    print(message)

def httpresponseheader(headerkey, headervalue):
    '''
    Fake setting the HTTP Response Header.  For example
    scraperwiki.utils.httpresponseheader('Content-Type', 'text/plain')
    '''
    pass

def GET():
    '''
    This is deprecated. Also, it's useless if you're
    running locally rather than as a CGI script.
    '''
    warnings.warn('Deprecated', DeprecationWarning)   

def scrape (url, params = None, user_agent = None) :
    '''
    Scrape a URL optionally with parameters.
    This is effectively a wrapper around urllib2.orlopen.
    '''

    headers = {}
    
    if user_agent:
        headers['User-Agent'] = user_agent
        
    data = params and urllib.urlencode(params) or None
    req = urllib2.Request(url, data=data, headers=headers)
    f = urllib2.urlopen(req)
    
    text = f.read()
    f.close()
    
    return text
            
def pdftoxml(pdfdata):
    """converts pdf file to xml file"""
    pdffout = tempfile.NamedTemporaryFile(suffix='.pdf')
    pdffout.write(pdfdata)
    pdffout.flush()

    xmlin = tempfile.NamedTemporaryFile(mode='r', suffix='.xml')
    tmpxml = xmlin.name # "temph.xml"
    cmd = 'pdftohtml -xml -nodrm -zoom 1.5 -enc UTF-8 -noframes "%s" "%s"' % (pdffout.name, os.path.splitext(tmpxml)[0])
    cmd = cmd + " >/dev/null 2>&1" # can't turn off output, so throw away even stderr yeuch
    os.system(cmd)

    pdffout.close()
    #xmlfin = open(tmpxml)
    xmldata = xmlin.read()
    xmlin.close()
    return xmldata

def swimport(name, swinstance="https://scraperwiki.com"):
    'Import from a ScraperWiki script'
    url = "%s/editor/raw/%s" % (swinstance, name)
    os.system('wget -O %s.py \'%s\'' % (name, url))
    return __import__(name)

def jsviewcall(name, **args):
    'callback to a view with parameter lists (cross language capability)'
    url = "https://scraperwiki.com/views/%s/run/?%s" % (name, urllib.urlencode(args))
    response = urllib.urlopen(url).read()
    try:
        return json.loads(response)
    except ValueError:
        return response

urllib2opener = None
def urllibSetup(http_proxy):
    raise NotImplementedError('Dunno what this does')

class SWImporter(object):
    def __init__(self, swinstance="https://scraperwiki.com"):
        raise NotImplementedError('Standard Python imports are used instead')



#!/usr/bin/env python2

# This file is part of Local Scraperlibs.

# Local Scraperlibs is free software: you can redistribute it and/or modify
# it under the terms of the GNU Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# Local Scraperlibs is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Public License for more details.

# You should have received a copy of the GNU Public License
# along with Local Scraperlibs.  If not, see <http://www.gnu.org/licenses/>.

import warnings
import os
from distutils.core import setup

def has_external_dependency(name):
    'Check that a non-Python dependency is installed.'
    for directory in os.environ['PATH'].split(':'):
        if os.path.exists(os.path.join(directory, name)):
            return True
    return False

if not has_external_dependency('wget'):
    warnings.warn(
        'wget not found: attach() and swimport() will not work.')

if not has_external_dependency('pdftohtml'):
    warnings.warn(
        'Local Scraperlibs requires pdftohtml, but pdftoxml was not found\n'
        'in the PATH. You probably need to install it.'
    )

setup(name='scraperwiki_local',
    author='Thomas Levine',
    author_email='thomas@scraperwiki.com',
    description='Local version of scraperwiki scraperlibs',
    url='https://github.com/tlevine/scraperwiki_local',
    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: SQL',
        'Topic :: Database :: Front-Ends',
    ],
    packages=['scraperwiki'],

    version = '0.0.2',
    license='GPL',
    install_requires = open('requirements.txt').read().split('\n')[:-1],
   )
